
/*
 *  Copyright (C) 1999  Wolfgang Zekoll  <wzk@um1.pce.de>
 *
 *  This software is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <sys/wait.h>


#include "lib.h"


#define	DEBUG(x)		


char	*program =		"";

int	configure_network =	0;


typedef struct _net {
    unsigned long number;

    unsigned long adr;
    unsigned long last;

    unsigned long mask;
    } net_t;


unsigned long atoip(char *string, char **r, int *error)
{
	int	i, k;
	unsigned long num;
	char	*p;

	*error = 0;
	num    = 0;

	p = string;
	for (i=0; i<4  &&  *p != 0; i++) {
		k = (unsigned) strtol(p, &p, 10);
		if (k < 0  ||  k > 255) {
			*error = 1;
			return (1);
			}

		num = (num << 8) + k;
		if (*p == '.') {
			p++;
			if (isdigit(*p) == 0) {
				*error = 1;
				return (1);
				}
			}
		else if (i != 3) {
			*error = 1;
			return (1);
			}
		}

	*r = p;
	return (num);
}

int read_ipnet(net_t *x, char *line, char **r)
{
	int	error;
	char	*p, *s;

	s = line;
	x->adr = x->number = atoip(skip_ws(line), &p, &error);
	if (error != 0) {
		fprintf (stderr, "%s: invalid ip number: %s\n", program, line);
		return (1);
		}
	else if (*p == '/') {
		p++;
		error = 0;
		if (strchr(p, '.') != NULL)
			x->mask = atoip(p, &p, &error);
		else {
			int	n;

			n = strtol(p, &p, 10);
			if (n < 0  ||  n > 32)
				error = 1;

			x->mask = (0xFFFFFFFFUL << (32 - n));
			}

		if (error != 0) {
			fprintf (stderr, "%s: invalid netmask: %s\n", program, line);
			return (1);
			}
			
		x->adr = x->adr & x->mask;
		}
	else
		x->mask = 0xFFFFFFFFUL;

	*r = p;
	if (*p != 0) {
		fprintf (stderr, "%s: extra characters behind specification: %s\n", program, line);
		return (1);
		}

	x->last = x->adr + (~ x->mask);
	return (0);
}

int print_ipnum(unsigned long ip)
{
	unsigned char *p;

	p = (unsigned char *) &ip;
	printf ("%u.%u.%u.%u\n", p[3], p[2], p[1], p[0]);

	return (0);
}

int print_iprange(net_t *net)
{
	unsigned long start, end, adr;
	unsigned char *p;

	adr = net->adr;
	p = (unsigned char *) &adr;

	if (net->last == net->adr) {
		start = net->adr;		/* x.x.x.x/32 - einzelner Host */
		end   = start + 1;
		}
	else if (net->last == net->adr + 1) {
		start = net->adr;		/* x.x.x.x/31 - 2er Netz */
		end   = start + 2;
		}
	else {
		start = net->adr + 1;
		end   = net->last;
		}
		
	for (adr = start; adr < end; adr++)
		printf ("%u.%u.%u.%u\n", p[3], p[2], p[1], p[0]);

	return (0);
}


void missing_arg(int c, char *string)
{
	fprintf (stderr, "%s: missing arg: -%c, %s\n", program, c, string);
	exit (-1);
}

int main(int argc, char *argv[])
{
	int	c, i, k;
	char	*p, option[80];
	net_t	net;
	
	program = argv[0];
	k = 1;

	while (k < argc  &&  argv[k][0] == '-'  &&  argv[k][1] != 0) {
		copy_string(option, argv[k++], sizeof(option));
		for (i=1; (c = option[i]) != 0; i++) {
			if (c == 'c') {
				if (k >= argc)
					missing_arg(c, "ip/mask");

				if (read_ipnet(&net, argv[k++], &p) == 0) {
					print_ipnum(net.number);
					print_ipnum(net.adr);
					print_ipnum(net.last);
					print_ipnum(net.mask);

					if (k < argc) {
						int	error;
						unsigned long host;
						
						for (i=k; i<argc; i++) {
							host = atoip(skip_ws(argv[i]), &p, &error);
							if (error != 0  ||  *p != 0) {
								fprintf (stderr, "%s: invalid ip nmber: %s\n", program, argv[i]);
								continue;
								}

							if (host > net.adr  &&  host < net.last)
								printf ("+ ");
							else
								printf ("- ");

							print_ipnum(host);
							}
						}
					}

				exit (0);
				}
			else {
				fprintf (stderr, "%s: unknown option: -%c\n", program, c);
				exit (-1);
				}
			}
		}

	if (k < argc) {
		for (i=k; i<argc; i++) {
			if (read_ipnet(&net, argv[i], &p) == 0)
				print_iprange(&net);
			}
		}
	else {
		char	word[80], line[300];

		while (fgets(line, sizeof(line), stdin) != NULL) {
			p = skip_ws(noctrl(line));
			if (*p == 0  ||  *p == '#')
				continue;

			get_word(&p, word, sizeof(word));
			if (read_ipnet(&net, word, &p) == 0)
				print_iprange(&net);
			}
		}
		
	exit (0);
}


